#! /bin/sh

# 1. Change the config options below.
# 2. Put some interesting queries in INFILE
# 3. Make sure the mysql daemon is running.
# 4. Run this script.
# 5. Enjoy the pdf file that was just created.

# Include functions for making LaTeX tables.
source tables.sh

### CONFIG OPTIONS:

# Output file name:
OUTFILE="./queries.tex"

# Input file name with your queries:
INFILE=queries

# Name of your database:
DB=yourdatabase

### FUNCTIONS:

# The MakeQuery function can be called like this:
#
#MakeQuery "SELECT name FROM members;" normal
#
# If the column names can't be extracted by the MakeQuery code then
# you need to supply those names manually:
#
#MakeQuery "SELECT * FROM members;" normal name address telephone
#
# Three columns would usually get the LaTeX format "|l|l|l|". If that
# gives problems---like a table that is too wide for the page---you
# can specify it yourself,e.g.:
#
#MakeQuery "SELECT * FROM members;" "|p{5cm}|p{4cm}|l|" name address telephone

MakeQuery(){
    QUERY=$1
    shift
    FORMAT=$1
    shift
    if test $# -ne 0; then
	# The user has specified the column names manually.
        COLS=$*
    else
	# Try to extract the correct column names
        COLS=$(echo "$QUERY" | sed "s/^SELECT //g;s/^DISTINCT//g;s/ FROM[[:print:]]*$//g;s/([^(]*)//g;s/,[^,]*AS//g;s/^[^,]*AS//g;s/_/\\\_/g;s/,//g")
    fi
    if test "x$FORMAT" = "xnormal"; then
	# Requested format is normal: no specific instructions on
	# width of columns. Specify the LaTeX format for the known
	# number of columns: "|l|l|...|l|"
        FORMAT="|"
        for COL in $COLS; do
            FORMAT="${FORMAT}l|"
        done
    fi

    # Print the query verbatim, but cut it into several lines if it is
    # too long
    cat <<EOF >> "$OUTFILE"

\begin{verbatim}
$(echo "$QUERY" | ./cutoff.sh 55)
\end{verbatim}

EOF

    # Count the number of columns:
    COUNT=0
    for item in $COLS; do
        COUNT=$(echo "scale=10; $COUNT+1" | bc -l)
    done

    # Now we finally have enough information to make a table. Format:

    # MakeTable database "command;" "caption" label "column format" \
    # column names

    MakeTable $DB "$QUERY;" none none "$FORMAT" $COLS
}

### MAIN:

UseOutfile "$OUTFILE"

# Make a new LaTeX report
cat <<EOF > "$OUTFILE"
\documentclass[12pt]{report}
\usepackage{longtable}
\setlongtables
\author{Maurits van Rees}
\title{Sample Database document}
\begin{document}
\chapter{Queries}

EOF


# Read the INFILE
while read line
do
    if test "${line:0:1}" = "#"; then
	# Ignore comment lines starting with '#': simply return true
        true;
    elif test "${line:0:2}" = "/*"; then
	# Print "/* comment lines */". Remove the "/*" and "/*" and
	# escape underscores.
        echo "$line" | sed "s/^\/\*//g;s/\*\/$//g;s/_/\\\_/g" >> $OUTFILE
    elif test ${#line} -eq 0; then
        # When you find an empty line... output an empty line. :)
        echo "" >> $OUTFILE
    else
	# This should be a query. Hurrah! Split it into a real query
	# and extra info that might have been supplied after a '#'
        REAL_QUERY=$(echo "$line" | sed "s/^\(.*\)#[[:blank:]*]\(.*\)$/\1/g")
        EXTRA_INFO=$(echo "$line" | grep "#" | sed "s/^\(.*\)#[[:blank:]*]\(.*\)$/\2/g")
        if test "x$EXTRA_INFO" = "x"; then
	    # There is no extra info, so treat it as normal
            EXTRA_INFO=normal
        fi
	# Now pass the info to the MakeQuery function.
        MakeQuery "$REAL_QUERY" $EXTRA_INFO
    fi
done <  "$INFILE"

# That's the end of the LaTeX report
cat <<EOF >> "$OUTFILE"
\end{document}
EOF

# Turn the created LaTeX file into PDF. Yes, you need to call this
# twice:
pdflatex "$OUTFILE"
pdflatex "$OUTFILE"
